function [param,sigp,stats, res, sigd ] = WEuler(vel,names)
% [param,sigp,stats] = WEuler(vel,names): Fits dNEU or Euler pole to velocity
% values.  Translation and rotation set parameters estimated.
% param values (upto 6) are determined by options.  
%   First three values are either dNEU or dXYZ (translation) and 
%   Second triplet is omega xyz (radians/Myr) 
% Parameter values are written to userdata of offset or align buttons in
% the form if 'dNEU NEU' of 'dXYZ XYX ELat Elong Erate' where ELat ELong
% are the latitude and longitude of the Euler pole (deg) and Erate is rate
% in deg/Myr (same as GLOBK frame_to_frame routine).
% Stats are statistics of fit with #site, N WRMS (mm/yr), N NRMS, E WRMS,
% WNRMS, U WRMS, U NRMS.
% res -- residuals 3xND (NEU order), sigd is data sigma
%
% Get options
ht = findobj(gcf,'Tag','EstTran'); hr = findobj(gcf,'Tag','EstRot');
et = get(ht,'Value'); er = get(hr,'Value');
%
% See what is to be estimated
np = 3 ;  % Always at least 3-parameters
if er == 1 && et == 1 , np = 6; end
param = zeros(np,1); sigp = zeros(np,1);
stats = zeros(1,7); % # sites, WMRS, NRMS in NEU
%
neq = zeros(np); bvec = zeros(np,1);
% Loop over the sites and increment normal equations
ns = length(vel(1,:));
if er == 0 && et == 2
    % Old WMean Code (done wiht vectors)
    for i = 1:3
        if i == 1, Vals = vel(4,:); Sig = vel(8,:);       % North
        elseif i == 2, Vals = vel(3,:); Sig = vel(7,:);   % East
        elseif i == 3, Vals = vel(10,:); Sig = vel(12,:); % Up
        end
        %
        % Form the weight vector
        w = 1./Sig.^2;
        Wmean = w*Vals'/sum(w);
        r = Vals - Wmean;
        if ns > 1, nrms = sqrt(sum(r.^2.*w)/(ns-1)); else nrms = 1; end
        wrms = nrms*sqrt(ns/sum(w));
        %
        param(i) = Wmean ; sigp(i) = wrms/sqrt(ns); 
        stats(1) = ns; 
        stats(2+2*(i-1)) = wrms; stats(3+2*(i-1)) = nrms;
    end
else
    % Standard estimation scheme for translation and rotations
    for k = 1:ns
       % Convert Position to XYZ and get rotation matrix from NEU to XYZ
       % Routine takes co-lat, long, height.
       clh = [pi/2-vel(2,k)*pi/180 vel(1,k)*pi/180 0]; 
       xyz = geod_to_xyz( clh ); 
       % Now convert get the rotation matrix from NEU to XYZ so that we can
       % rotate the partials from XYZ to NEU (where the correlation matrix
       % can be taken as diagonal almost).
       rot_mat = RotXYZ_to_NEU( clh ); 
       %
       % Now form partials of parameters with respect to vNEU
       neu_part = GenPart( xyz, rot_mat, np, er, et);
       dvel = [vel(4,k) ; vel(3,k) ; vel(10,k) ] ; % vNEU
       covv = zeros(3);
       covv(1,1) = vel(8,k)^2; covv(2,2) = vel(7,k)^2; covv(3,3) = vel(12,k)^2;
       covv(1,2) = vel(9,k)*vel(7,k)*vel(8,k); covv(2,1) = covv(1,2);
       W = eye(3)/covv; 
       neq = neq + neu_part'*W*neu_part ; bvec = bvec + neu_part'*W*dvel;
    end
    % Now finish up the solution
    covp = eye(np)/neq; param = covp*bvec; sigp = diag(covp).^0.5;
    % Output the results
%     for k = 1:np
%         fprintf(' %d  %8.5f +- %8.5f\n',k, param(k), sigp(k));
%     end
%     fprintf('Site        NRes +-       ERes +-        URes +-\n');
    r = zeros(3,ns); sigd = zeros(3,ns);
    for k = 1:ns
       clh = [pi/2-vel(2,k)*pi/180 vel(1,k)*pi/180 0]; 
       xyz = geod_to_xyz( clh ); 
       rot_mat = RotXYZ_to_NEU( clh ); 
       neu_part = GenPart( xyz, rot_mat, np, er, et);
       dvel = [vel(4,k) ; vel(3,k) ; vel(10,k) ] ; % vNEU
       sig = [vel(8,k) ; vel(7,k) ; vel(12,k) ] ; 
       r = dvel - neu_part*param ;
       res(:,k) = r ; sigd(:,k) = sig;
       %fprintf('%s %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f\n', names(k,:), ...
       %    res(1), vel(8,k), res(2), vel(7,k), res(3), vel(12,k));
    end
    % Now finish up the statistics
    chi = 0; 
    for k = 1:3
        if ns > 1, nrms = sqrt(sum(res(k,:).^2./sigd(k,:).^2)/(ns-1)); else nrms = 1; end
        wrms = nrms*sqrt(ns/sum(1./sigd(k,:).^2)); chi = chi + nrms^2;
        %
        stats(1) = ns; 
        stats(2+2*(k-1)) = wrms; stats(3+2*(k-1)) = nrms;
    end
    chi = sqrt(chi/3);
    sigp = sigp*chi;
end

% Now convert Wxyz to Euler pole with sigma.  This is just output to the
% screen.
if er == 1
    if chi < 1 , chi = 1; end
    wxyz = param(np-2:np); Cw = covp(np-2:np,np-2:np)*chi;
    rate = (wxyz'*wxyz)^0.5; 
    clh = [acos(wxyz(3)/rate) atan2(wxyz(2),wxyz(1)) rate];
    rot_mat = RotXYZ_to_NEU( clh ); 
    Ce = rot_mat*Cw*rot_mat'; 
    fprintf('--------------------------------------------------\n');
    fprintf('Euler Pole Estimated with %d sites, Chi %6.2f\n',ns,chi);
    fprintf('Latitude  %9.3f  %8.3f deg \n',(pi/2-clh(1))*180/pi, sqrt(Ce(1,1))*180/pi);
    fprintf('Longitude %9.3f  %8.3f deg \n',(clh(2))*180/pi,      sqrt(Ce(2,2))*180/pi);
    fprintf('Rate      %9.6f  %8.6f deg/Myr \n',clh(3)*180/pi/1000, sqrt(Ce(3,3))*180/pi/1000);
    fprintf('Corr: LatLong %7.3f LatRate %7.3f LongRate %7.3f\n', ...
        Ce(1,2)/sqrt(Ce(1,1)*Ce(2,2)), Ce(1,3)/sqrt(Ce(1,1)*Ce(3,3)), ...
        Ce(2,3)/sqrt(Ce(2,2)*Ce(3,3)));
end
    
    
       
       
       
       